<?php
// $Id: classroom.admin.inc,v 1.1 2009/07/27 14:14:06 osoh Exp $

/**
 * Courses admin tab
 */
function classroom_courses_admin() {
  $form = array();

  $form['classroom_course_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Type'),
    '#description' => t('Content types that could be used as courses.'),
    '#default_value' => variable_get('classroom_course_types', array()),
    '#options' => _classroom_get_node_types(),
  );

  return system_settings_form($form);
}

/**
 * Course validation
 */
function classroom_courses_admin_validate($form, $form_state) {
  //Get the values that are checked and put in an array
  $values = array_unique(array_values($form_state['values']['classroom_course_types']));
  $resource_types = array_unique(array_values(variable_get('classroom_resource_types',array())));
  $assignment_types = array_unique(array_values(variable_get('classroom_assignment_types', array())));
  $response_types = array(variable_get('classroom_response_types', array()));
 
  //Removes the empty element
  $values = array_filter($values);
  $assignment_types = array_filter($assignment_types);
  $response_types = array_filter($response_types);

  foreach($values as $val) {
    if ((in_array($val, $resourse_types)) ||
     (in_array($val, $assignment_types)) ||
     (in_array($val, $response_types))) {
       form_set_error('classroom_courses_types', t('Cannot overlap the types for the elements of Classroom module'));
     }
  } 
}


/**
 * Assignments admin tab
 */
function classroom_assignments_admin() {
  $form = array();

  $form['classroom_assignment_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Type'),
    '#description' => t('Content types that could be used as assignments.'),
    '#default_value' => variable_get('classroom_assignment_types', array()),
    '#options' => _classroom_get_node_types(),
  );

  return system_settings_form($form);
}

/**
 * Assignments admin validation
 */
function classroom_assignments_admin_validate($form, $form_state) {
  //Get the values that are checked and put in an array
  $values = array_unique(array_values($form_state['values']['classroom_assignment_types']));
  $course_types = array_unique(array_values(variable_get('classroom_course_types',array())));
  $resource_types = array_unique(array_values(variable_get('classroom_resource_types', array())));
  $response_types = array(variable_get('classroom_response_types', array()));

 
  //Removes the empty element
  $values = array_filter($values);
  $course_types = array_filter($course_types);
  $resource_types = array_filter($resource_types);
  $response_types = array_filter($response_types);
  
  foreach($values as $val) {
    if ((in_array($val, $course_types)) ||
     (in_array($val, $resource_types)) ||
     (in_array($val, $response_types))) {
       form_set_error('classroom_resource_types', t('Cannot overlap the types for the elements of Classroom module'));
     }
  } 
}


/**
 * Response admin tab
 */
function classroom_responses_admin() {
  $form = array();

  $form['classroom_response_types'] = array(
    '#type' => 'radios',
    '#title' => t('Type'),
    '#description' => t('Content types that could be used as responses.'),
    '#default_value' => variable_get('classroom_response_types', array()),
    '#options' => _classroom_get_node_types(),
  );

  return system_settings_form($form);
}

/**
 * Response admin validation
 */
function classroom_responses_admin_validate($form, $form_state) {
  //Get the values that are checked and put in an array
  $values = $form_state['values']['classroom_response_types'];
  $course_types = array_unique(array_values(variable_get('classroom_course_types',array())));
  $assignment_types = array_unique(array_values(variable_get('classroom_assignment_types', array())));
  $resource_types = array_unique(array_values(variable_get('classroom_resource_types', array())));

  if ($values == null) return;
  //Removes the empty element
  $course_types = array_filter($course_types);
  $assignment_types = array_filter($assignment_types);
  $resource_types = array_filter($resource_types);
  
  if ((in_array($values, $course_types)) ||
    (in_array($values, $assignment_types)) ||
    (in_array($values, $resource_types))) {
      form_set_error('classroom_response_types', t('Cannot overlap the types for the elements of Classroom module'));
    }
}

/**
 * Resources admin tab page
 */
function classroom_resources_admin() {
  $form = array();

  $form['classroom_resource_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Type'),
    '#description' => t('Content types that could be used as resources.'),
    '#default_value' => variable_get('classroom_resource_types', array()),
    '#options' => _classroom_get_node_types(),
  );

  return system_settings_form($form);
}

/**
 * Resource validation
 */
function classroom_resources_admin_validate($form, $form_state) {
  //Get the values that are checked and put in an array
  $values = array_unique(array_values($form_state['values']['classroom_resource_types']));
  $course_types = array_unique(array_values(variable_get('classroom_course_types',array())));
  $assignment_types = array_unique(array_values(variable_get('classroom_assignment_types', array())));
  $response_types = array_unique(array_values(variable_get('classroom_response_types', array())));

 
  //Removes the empty element
  $values = array_filter($values);
  $course_types = array_filter($course_types);
  $assignment_types = array_filter($assignment_types);
  $response_types = array_filter($response_types);

  foreach($values as $val) {
    if ((in_array($val, $course_types)) ||
     (in_array($val, $assignment_types)) ||
     (in_array($val, $response_types))) {
       form_set_error('classroom_resource_types', t('Cannot overlap the types for the elements of Classroom module'));
     }
  } 
}
