<?php
// $Id: classroom.resources.inc,v 1.1 2009/07/27 14:14:06 osoh Exp $

/**
 * Get resources associated to a given course.
 *
 * @param $course
 *   Course to query about her resources.
 * @return
 *   An array containing resource objects associated with $course. Returns an
 *   empty array if there's no resources.
 */
function classroom_get_resources_for($course) {
  $result = db_query("SELECT resource_nid FROM {classroom_course_resource} WHERE course_nid = %d", $course);
  $resources = array();
  while ($row = db_fetch_array($result)) {
    // OPTIMIZE
    $resources[] = node_load($row['resource_nid']);
  }
  return $resources;
}

/**
 * Add a resource to a course.
 *
 * @param $course
 *   Course to add the resource.
 * @param $resource
 *   Resource to add.
 * @return
 *   This function returns true if the resource was added to the course
 *   successfully. Otherwise it returns false.
 */
function classroom_add_resource($course, $resource) {
  if (isset($course->nid) and isset($resource->nid)) {
    db_query("INSERT INTO {classroom_course_resource} (resource_nid, course_nid) VALUES (%d, %d)",
      $resource->nid, $course->nid);
    return (db_error() == 0);
  } else {
    return FALSE;
  }
}

/**
 * Remove a resource from a course.
 *
 * @param $course
 *   Course to remove the resource from.
 * @param $resource
 *   Resource to remove.
 * @return
 *   This function returns true if the resource was removed from the course
 *   successfully. Otherwise it returns false.
 *   FIXME: It returns true if the resource and the course weren't associated.
 */
function classroom_remove_resource_from($course, $resource) {
  if (isset($course->nid) and isset($resource->nid)) {
    db_query("DELETE FROM {classroom_course_resource} WHERE resource_nid = %d AND course_nid = %d",
      $resource->nid, $course->nid);
    return (db_error() == 0);
  } else {
    return FALSE;
  }
}

/**
 * Implementation of hook_nodeapi
 */
function _classroom_resource_nodeapi(&$node, $op, $a3 = NULL, $a4 = NULL) {
  switch ($op){
  case 'view':
  case 'alter':
    return _classroom_resource_node_breadcrumb($node);
    break;
  }
}

/**
 * Establish the breadcrumb for the resources
 */
function _classroom_resource_node_breadcrumb(&$node) {

  $course_node = _classroom_resource_get_course($node); 
  $breadcrumb = array(
    l(t("Home"), "<front>"),
    l(t("Courses"), "classroom/"),
    l($course_node->title, "classroom/course/".$course_node->nid),
    l(t("Resources"), "classroom/course/".$course_node->nid."/resources"),
  );

  drupal_set_breadcrumb($breadcrumb);
}

/**
 *  Obtain the course associated to a resource node
 */
function _classroom_resource_get_course( $resource ) {
  $result = db_query("SELECT node.* FROM {node},{classroom_course_resource} WHERE course_nid = nid and resource_nid = %d",
    $resource->nid);
  return db_fetch_object($result);
}
