<?php
// $Id: classroom_signup.pages.inc,v 1.1 2009/07/27 14:14:06 osoh Exp $

/**
 * Returns a list of sign up requests for a course.
 * TODO: refactor query into a function classroom_get_courses.
 * TODO: allow filtering by date capabilities.
 * @param $course_nid
 *   Course ID.
 */
function classroom_signup_requests_page($course_nid) {
  $headers = array(
    'name' => array('data' => t('Name'), 'field' => 'name'),
    'datetime' => array('data' => t('Date/Time'), 'field' => 'created'),
    'status' => array('data' => t('Status')),
  );

  // FIXME: it must call a function to retrieve data.
  $qry = "SELECT name, rid, {classroom_signup_request}.created, {classroom_signup_request}.status as status FROM {classroom_signup_request} LEFT JOIN {users} USING (uid) WHERE course_nid = %d";
  $pager_limit = variable_get('classroom_pager_limit', CLASSROOM_PAGER_LIMIT_DEFAULT);
  $result = pager_query($qry, $pager_limit, 0, NULL, $course_nid);

  $rows = array();
  while ($request = db_fetch_object($result)) {
    $row = array();
    $row['user'] = l($request->name, "node/$course_nid");
    $row['created'] = format_date($request->created, 'small');
    $status_label = _classroom_signup_status_label($request->status);
    if ($request->status == CLASSROOM_SIGNUP_STATUS_PENDING) {
      $row['status'] = l($status_label,
        "classroom/course/$course_nid/request/$request->rid/answer");
    } else {
      $row['status'] = $status_label;
    }
    $rows[] = $row;
  }

  if (empty($rows)) {
    $output = "<p>" . t("There are no requests for this course.") . "</p>";
  } else {
    $output = theme('table', $headers, $rows);
    $output .= theme('pager');
  }

  return $output;
}

/**
 * Form to sign up in a course.
 *
 * @param $form
 *   Form definition (Drupal style).
 * @param $course_nid
 *   Course ID.
 */
function classroom_signup_request_form($form, $course_nid)
{
  global $user;

  $form = array();

  $form['request_comment'] = array(
    '#type' => 'textarea',
    '#value' => '',
    '#description' => t('Comment for the requester to read (if any).'),
    '#rows' => 3,
  );

  $form['course_nid'] = array(
    '#type' => 'hidden',
    '#value' => $course_nid,
  );

  $form['#redirect'] = array("classroom/course");

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  return $form;

}

/**
 * Form to sign up in a course. (hook_form_submit() implementation).
 *
 * @param $form
 *   Form definition (Drupal style).
 * @param $form_state
 *   Form state (Drupal style).
 */
function classroom_signup_request_form_submit($form, $form_state)
{
  global $user;

  $course_nid = $form['#post']['course_nid'];
  $course = node_load($course_nid);
  classroom_signup_request($user, $course);
}

/**
 * Form to answer a sign up request.
 *
 * @param $form
 *   Form definition (Drupal style).
 * @param $rid
 *   Request ID.
 */
function classroom_signup_answer_form($form, $rid) {
  $request = classroom_signup_get_request_by_rid($rid);

  $user = user_load(array('uid' => $request->uid));

  $course_node = node_load($request->course_nid);
  $breadcrumb = array(
    l(t("Home"), "<front>"),
    l(t("Courses"), "classroom/"),
    l($course_node->title, "classroom/course/".$course_node->nid),
    l(t("Sign up requests"), "classroom/course/".$course_node->nid."/requests"),
  );

  drupal_set_breadcrumb($breadcrumb);

  $form = array();

  $form['rid'] = array(
    '#type' => 'hidden',
    '#value' => $rid
  );

  $form['user'] = array(
    '#type' => 'markup',
    '#value' => "<p>$user->name</p>",
    '#description' => t('Comment for the requester to read (if any).'),
    '#rows' => 3,
  );

  $form['response_comment'] = array(
    '#type' => 'textarea',
    '#value' => $request->response_comment,
    '#description' => t('Comment for the requester to read (if any).'),
    '#rows' => 3,
  );

  $form['accept'] = array(
    '#type' => 'submit',
    '#value' => t('Accept'),
  );

  $form['deny'] = array(
    '#type' => 'submit',
    '#value' => t('Reject'),
  );

  $form['#redirect'] = array("classroom/course/{$request->course_nid}/requests");

  return $form;
}

/**
 * Form to answer a sign up request. (hook_form_submit() implementation).
 *
 * @param $form
 *   Form definition (Drupal style).
 * @param $form_state
 *   Form state (Drupal style).
 */
function classroom_signup_answer_form_submit($form, $form_state) {
  $request = classroom_signup_get_request_by_rid($form['#post']['rid']);
  $request->response_comment = $form['#post']['response_comment'];
  switch ($form['#post']['op']) {
  case t('Accept'):
    classroom_signup_accept_request($request);
    break;
  case t('Reject'):
    classroom_signup_reject_request($request);
    break;
  }
}
